#include <stdio.h>
#include <stdlib.h>
#include "automata.h"
#include "config.h"

Automata::Automata() {

}

void Automata::add_transition(int state, int new_state, const char *s) {
	TRANSITION trans = {state, new_state, string(s) };
	this->transitions.push_back(trans);
}

void Automata::print() {
	puts("AUTOMATA:");
	for(unsigned int i=0; i < this->transitions.size(); i++) {
		TRANSITION trans = this->transitions[i];
		printf("%d -> %d: %s\n", trans.state, trans.new_state, trans.s.c_str());
	}
}

unsigned int Automata::size() {
	return this->transitions.size();
}

int Automata::state() {
	return this->current_state;
}

void Automata::start(int state) {
	this->current_state = state;
}

// Applies to jump to new state
// Returns the type of string identified according to the configuration
int Automata::apply(const char *s) {
	for(unsigned int i=0; i < this->transitions.size(); i++) {
		TRANSITION trans = this->transitions[i];
		
		//printf("STATE: %d -- %d\n",trans.state,this->current_state);
		// If satisfies the transition
		if(trans.state == this->current_state) {
			// If it is an empty transition or if it matches
			if(trans.s.size() == 0 || trans.s.compare(s) == 0) {
				// Jumps to the new state
				this->current_state = trans.new_state;
				return NUM_CONFIG_SECTIONS;		// Didn't match with any configuration, but with a simple transition
			}
			
			// Searches to match with a configuration section
			CONFIG_SECTION section = find_section(trans.s.c_str());
			if(check_string(section, s)) {
				// Jumps to the new state
				this->current_state = trans.new_state;
				return section;
			}
		}
	}
	
	return -1;	// Error: Didn't match
}
