#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <iostream>

#include "config.h"

#define SIZE_BUFFER		1000

using namespace std;

vector<string> config_sections[NUM_CONFIG_SECTIONS];
Automata automata;


CONFIG_SECTION find_section(const char *s) {
	if(strcmp(s,"[open_session]") == 0)
		return OPEN_SESSION;
	if(strcmp(s,"[close_session]") == 0)
		return CLOSE_SESSION;
	if(strcmp(s,"[space_br]") == 0)
		return SPACE_BR;
	if(strcmp(s,"[new_party]") == 0)
		return NEW_PARTY;
	if(strcmp(s,"[new_deputy]") == 0)
		return NEW_DEPUTY;
	
	return NUM_CONFIG_SECTIONS;	// ERROR
}

const char *str_section(const int section) {
	switch(section) {
		case OPEN_SESSION:					
			return "[open_session]";
		case CLOSE_SESSION:
			return "[close_session]";
		case SPACE_BR:
			return "[space_br]";
		case NEW_PARTY:
			return "[new_party]";
		case NEW_DEPUTY:
			return "[new_deputy]";
		case NUM_CONFIG_SECTIONS:
			return "SIMPLE MATCH";
	}
	return "UNRECOGNIZED";	// ERROR
}

void read_config(const char *file) {
	CONFIG_SECTION current_section = NUM_CONFIG_SECTIONS;
	char buf[SIZE_BUFFER];
	FILE *conf = fopen(file, "rt");
	
	if(conf == NULL) {
		fprintf(stderr, "Cannot open configuration file '%s'\n",file);
		exit(EXIT_FAILURE);	// Exit with error
	}
	fputs("Reading configuration...\n",stderr);
	
	while( fgets(buf, SIZE_BUFFER, conf) != NULL ) {
		int len = strlen(buf);

		// Removes the new line in the end		
		if(buf[len-1]=='\n')
			buf[--len]='\0';
		if(buf[len-1]=='\r')
			buf[--len]='\0';
		
		if(len < 1)	// Empty line
			continue;
		if(buf[0]=='#' || buf[0]==';')
			continue;	// Comment
		
		// Automata description
		if(strcmp(buf,"[general]") == 0) {
			int n_trans, start_state;
			fgets(buf, SIZE_BUFFER, conf);
			sscanf(buf," %d %d",&n_trans,&start_state);
			fprintf(stderr,"Reading automata (%d transitions)...\n",n_trans);
			
			// Reads the automata
			for(int i=0;i<n_trans;i++) {
				fgets(buf, SIZE_BUFFER, conf);
				
				int state, new_state;
				char s[SIZE_BUFFER]  = "\0";
				sscanf(buf, " %d %d %s", &state, &new_state, s);
				// adds new the new transition to the automata
				automata.add_transition(state, new_state, s);
			}
			
			automata.start(start_state);
			//automata.print();
			continue;
		}
		
		
		// Searches for which section it is
		CONFIG_SECTION tmp_section = find_section(buf);
		if(tmp_section < NUM_CONFIG_SECTIONS) {	// If it is found,
			current_section = tmp_section;		// new section is applied
			continue;
		}
		
		// Adds the string to the configuration
		if(current_section >= NUM_CONFIG_SECTIONS)
			fprintf(stderr,"Some errors found while loading configuration file: %s\n",buf);
		else
			config_sections[current_section].push_back(string(buf));
		
		//puts(buf);
	}
}

bool check_string(CONFIG_SECTION section, const char *s) {
	if(!(section < NUM_CONFIG_SECTIONS))
		return false;
	
	// If it is empty, matches with anything
	if(config_sections[section].size() == 0)
		return true;
	
	for(unsigned int i=0; i<config_sections[section].size(); i++) {
		string orig_s = config_sections[section][i];
		
		if(orig_s.compare(0, orig_s.size(), s, 0, orig_s.size()) == 0)
			return true;
	}
	return false;
}

void print_configuration() {
	puts("CONFIGURATION:");
	for(int i=0; i<NUM_CONFIG_SECTIONS; i++) {
		printf("%s\n",str_section(i));
		for(unsigned int j=0;j<config_sections[i].size();j++)
			cout << config_sections[i][j] << endl;
		putchar('\n');
	}
}
